
Texture2D ShaderTexture : register(t0);
SamplerState Sampler : register(s0);

struct VS_IN
{
	float4 pos : SV_Position;
	float2 uv : TEXCOORD;
};

struct PS_IN
{
	float4 pos : SV_Position;
	float2 uv : TEXCOORD;
	//float4 col : COLOR;
};

cbuffer cbPerFrameData : register(b0) {
	float4 HmdWarpParam;
	float2 Scale;
	float2 ScaleIn;
	float2 LensCenter;

	float2 offset;
}
cbuffer cbPerFrameData : register(b0) {
	float2 rotationSinCos;
}

float2 HmdWarp(float2 in01, float2 in02)
{
	//float4 HmdWarpParam = float4(1, 0.22, 0.14, 0);
	//float2 Scale = float2(0.5, 1);
	//float2 ScaleIn = float2(1, 1);
	//float2 LensCenter = float2(0.5, 0.5);

	float2 theta = (in01 - LensCenter) * ScaleIn;
	float  rSq = theta.x * theta.x + theta.y * theta.y;
	float2 theta1 =
		theta * ((1.0f + in02.x) + in02.y * rSq) // correct chromatic aberr.
		* (HmdWarpParam.x +       // correct lens distortion
			HmdWarpParam.y * rSq +
			HmdWarpParam.z * rSq * rSq +
			HmdWarpParam.w * rSq * rSq * rSq);

	theta1 = (Scale * theta1) + LensCenter;
	theta1.x = theta1.x - (LensCenter.x - 0.25f);
	theta1.y = theta1.y - (LensCenter.y - 0.5f);


	// We're sampling from the full texture not a half texture like in libovr
	theta1.x *= 2.0f;
	//theta1.x = theta1.x + ViewportXOffset;
	//theta1.y = theta1.y - ViewportYOffset;
	return theta1;

	//float2x2 rotationMatrix = float2x2(rotationSinCos.y, -rotationSinCos.x, rotationSinCos.x, rotationSinCos.y);

	//theta1 -= 0.5;
	//float2 result = mul(theta1, rotationMatrix);
	//result += 0.5;
	//return result;
}



PS_IN VS( VS_IN input )
{
	PS_IN output = (PS_IN)0;
	
	//float rotationAngle = 45;

	//float s = sin(rotationAngle);
	//float c = cos(rotationAngle);
	//float2x2 rotationMatrix = float2x2(c, -s, s, c);

	float2x2 rotationMatrix = float2x2(rotationSinCos.y, -rotationSinCos.x, rotationSinCos.x, rotationSinCos.y);

	//input.pos -= float4(0.5f, 0,0,0);
	//float2 rot = mul(float2(input.pos.x, input.pos.y), rotationMatrix);
	//input.pos = float4(rot.x, rot.y, input.pos.z, input.pos.w);
	//input.pos += float4(0.5f, 0,0,0);
	output.pos = input.pos;// / float4(640, 480, 1, 1);

	input.uv -= float2(0.5f, 0.5);
	input.uv = mul(input.uv, rotationMatrix);
	input.uv += float2(0.5f, 0.5);
	output.uv = input.uv;

	//output.col = float4(1, shaderVal, 0, 1);
	
	return output;
}

float4 PS( PS_IN input ) : SV_Target
{
	return ShaderTexture.Sample(Sampler, input.uv);
	float2 tcRed;
	float2 tcGreen;
	float2 tcBlue;
	float4 outColor;
	float4 chroma = float4(0,0,0,0);

	//warp, complete source at https://github.com/cybereality/Perception/blob/master/Release/Perception/fx/OculusRift.fx

	tcRed = HmdWarp(input.uv, float2(chroma.x, chroma.y));
	tcGreen = HmdWarp(input.uv, float2(0, 0));
	tcBlue = HmdWarp(input.uv, float2(chroma.z, chroma.w));

	float width;
	float height;
	ShaderTexture.GetDimensions(width, height);
	float texelWidth = 1 / width;

	outColor.r = ShaderTexture.Sample(Sampler, tcRed.xy).r;
	outColor.g = ShaderTexture.Sample(Sampler, tcGreen.xy).g;
	outColor.b = ShaderTexture.Sample(Sampler, tcBlue.xy).b;

	return outColor;
}

technique10 Render
{
	pass P0
	{
		SetGeometryShader( 0 );
		SetVertexShader( CompileShader( vs_4_0, VS() ) );
		SetPixelShader( CompileShader( ps_4_0, PS() ) );
	}
}
