/*
Copyright (c) 2019 Odd Sheep SL

This work is licensed under the Creative Commons
Attribution-NonCommercial-ShareAlike 4.0 International License.
To view a copy of this license, visit
http://creativecommons.org/licenses/by-nc-sa/4.0/

Created by Xavier Selva for Odd Sheep SL (Trinus PSVR)

There are 4 UI pages that can be toggled for different UI states
Each page is toggled with keys 1, 2, 3, 4 (page 1 is enabled by default)

moveUIRects will move source pixel rect to dest rect, suitable for simple UIs
	rounding parameter not working yet (should allow circle shapes)

moveUIMask will move pixels that match the mask, with given position and scale
(Red is page0, Green is page1, Blue is page2, Alpha is page3)
for example (0.5, 1, 0, 1) is a pixel to show with page0 active, at half transparency and page1 opaque (1 = opaque)
this alpha is multiplied by the global alpha
note that page 3 (taken from alpha channel) is inverted (1 = transparent)

monoSBS is a simple (pixel) shader to duplicate the source input (side by side), to use if Depth3D is disabled (as a lightweight alternative)

alpha parameter sets an alpha for the moved ui, to reduce interference with game

instructions for preset creation can be found at:
https://www.trinusvirtualreality.com/vr-conversion-for-non-vr-games

this shader has been originally created to work with Trinus PSVR, to improve UI when converting non-VR PC games
visit trinusvr.com for more information
*/
//TODO fix rounding (not too important, mask option should sort that out)
//TODO add keys to move position of mask (so user can customise a bit in realtime), maybe just one key pressed + mouse delta?
//problem is we cannot persist the changed pos and scale at shader level (can we?)... this would need to happen on the cpu side


uniform float alpha <ui_min = 0; ui_max = 1; ui_step = 0.01; ui_type = "drag"; ui_tooltip = "Alpha"; > = 0.75;
uniform float2 maskSrcOffset <ui_min = -1; ui_max = 1; ui_step = 0.0001; ui_type = "slider"; ui_tooltip = "Offset applied to the masked source pixels, applies to all pages"; > = float2(0, 0);
uniform float2 maskDstOffset <ui_min = -1; ui_max = 1; ui_step = 0.0001; ui_type = "slider"; ui_tooltip = "Offset applied to UI page position, applies to all pages."; > = float2(0, 0);

uniform float2 maskPos0 <ui_min = -1; ui_max = 1; ui_step = 0.0001; ui_category = "Mask Page0"; ui_type = "slider"; ui_tooltip = "Destination for masked UI (Red)"; >;
uniform float2 maskScale0 <ui_min = 0; ui_max = 2; ui_step = 0.0001; ui_category = "Mask Page0"; ui_type = "slider"; ui_tooltip = "UI (Red) Scale (w,h)"; > = float2(1, 1);

uniform float2 maskPos1 <ui_min = -1; ui_max = 1; ui_step = 0.0001; ui_category = "Mask Page1"; ui_type = "slider"; ui_tooltip = "Destination for masked UI (Green)"; >;
uniform float2 maskScale1 <ui_min = 0; ui_max = 2; ui_step = 0.0001; ui_category = "Mask Page1"; ui_type = "slider"; ui_tooltip = "UI (Green) Scale (w,h)"; > = float2(1, 1);

uniform float2 maskPos2 <ui_min = -1; ui_max = 1; ui_step = 0.0001; ui_category = "Mask Page2"; ui_type = "slider"; ui_tooltip = "Destination for masked UI (Blue)"; >;
uniform float2 maskScale2 <ui_min = 0; ui_max = 2; ui_step = 0.0001; ui_category = "Mask Page2"; ui_type = "slider"; ui_tooltip = "UI (Blue) Scale (w,h)"; > = float2(1, 1);

uniform float2 maskPos3 <ui_min = -1; ui_max = 1; ui_step = 0.0001; ui_category = "Mask Page3"; ui_type = "slider"; ui_tooltip = "Destination for masked UI (Alpha)"; >;
uniform float2 maskScale3 <ui_min = 0; ui_max = 2; ui_step = 0.0001; ui_category = "Mask Page3"; ui_type = "slider"; ui_tooltip = "UI (Alpha) Scale (w,h)"; > = float2(1, 1);

uniform float4 srcRect0A <ui_min = 0; ui_max = 1; ui_step = 0.001; ui_category = "Rects Page 0"; ui_type = "slider"; ui_tooltip = "Source Rect(x,y,w,h)"; > = float4(0, 0, 1, 1);
uniform float4 dstRect0A <ui_min = 0; ui_max = 1; ui_step = 0.001; ui_category = "Rects Page 0"; ui_type = "slider"; ui_tooltip = "Dest Rect(x,y,w,h)";  > = float4(0, 0, 1, 1);
uniform float rounding0A <ui_min = 0; ui_max = 1; ui_step = 0.01;  ui_category = "Rects Page 0"; ui_type = "slider"; ui_tooltip = "Rounds the rect. Range 0..1, 1 being full circle/ellipse"; >;

uniform float4 srcRect0B <ui_min = 0; ui_max = 1; ui_step = 0.001; ui_category = "Rects Page 0"; ui_type = "slider"; ui_tooltip = "Second Source Rect(x,y,w,h)"; > = float4(0, 0, 1, 1);
uniform float4 dstRect0B <ui_min = 0; ui_max = 1; ui_step = 0.001; ui_category = "Rects Page 0"; ui_type = "slider"; ui_tooltip = "Second Dest Rect(x,y,w,h)";  > = float4(0, 0, 1, 1);
uniform float rounding0B <ui_min = 0; ui_max = 1; ui_step = 0.01;  ui_category = "Rects Page 0"; ui_type = "slider"; ui_tooltip = "Rounds the rect. Range 0..1, 1 being full circle/ellipse"; >;

uniform float4 srcRect1A <ui_min = 0; ui_max = 1; ui_step = 0.001; ui_category = "Rects Page 1"; ui_type = "slider"; ui_tooltip = "Source Rect(x,y,w,h)"; > = float4(0, 0, 1, 1);
uniform float4 dstRect1A <ui_min = 0; ui_max = 1; ui_step = 0.001; ui_category = "Rects Page 1"; ui_type = "slider"; ui_tooltip = "Dest Rect(x,y,w,h)";  > = float4(0, 0, 1, 1);
uniform float rounding1A <ui_min = 0; ui_max = 1; ui_step = 0.01;  ui_category = "Rects Page 1"; ui_type = "slider"; ui_tooltip = "Rounds the rect. Range 0..1, 1 being full circle/ellipse"; >;

uniform float4 srcRect1B <ui_min = 0; ui_max = 1; ui_step = 0.001; ui_category = "Rects Page 1"; ui_type = "slider"; ui_tooltip = "Second Source Rect(x,y,w,h)"; > = float4(0, 0, 1, 1);
uniform float4 dstRect1B <ui_min = 0; ui_max = 1; ui_step = 0.001; ui_category = "Rects Page 1"; ui_type = "slider"; ui_tooltip = "Second Dest Rect(x,y,w,h)";  > = float4(0, 0, 1, 1);
uniform float rounding1B <ui_min = 0; ui_max = 1; ui_step = 0.01;  ui_category = "Rects Page 1"; ui_type = "slider"; ui_tooltip = "Rounds the rect. Range 0..1, 1 being full circle/ellipse"; >;

uniform float4 srcRect2A <ui_min = 0; ui_max = 1; ui_step = 0.001; ui_category = "Rects Page 2"; ui_type = "slider"; ui_tooltip = "Source Rect(x,y,w,h)"; > = float4(0, 0, 1, 1);
uniform float4 dstRect2A <ui_min = 0; ui_max = 1; ui_step = 0.001; ui_category = "Rects Page 2"; ui_type = "slider"; ui_tooltip = "Dest Rect(x,y,w,h)";  > = float4(0, 0, 1, 1);
uniform float rounding2A <ui_min = 0; ui_max = 1; ui_step = 0.01;  ui_category = "Rects Page 2"; ui_type = "slider"; ui_tooltip = "Rounds the rect. Range 0..1, 1 being full circle/ellipse"; >;

uniform float4 srcRect2B <ui_min = 0; ui_max = 1; ui_step = 0.001; ui_category = "Rects Page 2"; ui_type = "slider"; ui_tooltip = "Second Source Rect(x,y,w,h)"; > = float4(0, 0, 1, 1);
uniform float4 dstRect2B <ui_min = 0; ui_max = 1; ui_step = 0.001; ui_category = "Rects Page 2"; ui_type = "slider"; ui_tooltip = "Second Dest Rect(x,y,w,h)"; > = float4(0, 0, 1, 1);
uniform float rounding2B <ui_min = 0; ui_max = 1; ui_step = 0.01;  ui_category = "Rects Page 2"; ui_type = "slider"; ui_tooltip = "Rounds the rect. Range 0..1, 1 being full circle/ellipse"; >;

uniform float4 srcRect3A <ui_min = 0; ui_max = 1; ui_step = 0.001; ui_category = "Rects Page 3"; ui_type = "slider"; ui_tooltip = "Source Rect(x,y,w,h)"; > = float4(0, 0, 1, 1);
uniform float4 dstRect3A <ui_min = 0; ui_max = 1; ui_step = 0.001; ui_category = "Rects Page 3"; ui_type = "slider"; ui_tooltip = "Dest Rect(x,y,w,h)";  > = float4(0, 0, 1, 1);
uniform float rounding3A <ui_min = 0; ui_max = 1; ui_step = 0.01; ui_category = "Rects Page 3"; ui_type = "slider"; ui_tooltip = "Rounds the rect. Range 0..1, 1 being full circle/ellipse"; >;

uniform float4 srcRect3B <ui_min = 0; ui_max = 1; ui_step = 0.001; ui_category = "Rects Page 3"; ui_type = "slider"; ui_tooltip = "Second Source Rect(x,y,w,h)"; > = float4(0, 0, 1, 1);
uniform float4 dstRect3B <ui_min = 0; ui_max = 1; ui_step = 0.001; ui_category = "Rects Page 3"; ui_type = "slider"; ui_tooltip = "Second Dest Rect(x,y,w,h)"; > = float4(0, 0, 1, 1);
uniform float rounding3B <ui_min = 0; ui_max = 1; ui_step = 0.01; ui_category = "Rects Page 3"; ui_type = "slider"; ui_tooltip = "Rounds the rect. Range 0..1, 1 being full circle/ellipse"; >;

//Trinus edits the shader before installing into the game, replacing %%pagenKeyMap with an actual keycode like 0x31
//Note that Trinus will overwrite changes done to the shader next time you install ReShade into the game (losing shader edits)
uniform bool page0 < source = "key"; keycode = 0x31; mode = "toggle"; >;
uniform bool page1 < source = "key"; keycode = 0x32; mode = "toggle"; >;
uniform bool page2 < source = "key"; keycode = 0x33; mode = "toggle"; >;
uniform bool page3 < source = "key"; keycode = 0x34; mode = "toggle"; >;
//uniform bool mouseChangeMaskPos < source = "key"; keycode = %%mousePosKeyMap; mode = "press"; >;
//uniform bool mouseChangeMaskScale < source = "key"; keycode = %%mouseScaleKeyMap; mode = "press"; >;

uniform float2 mousedelta < source = "mousedelta"; >;

texture backBufferTex : COLOR;
sampler Sampler
{
	Texture = backBufferTex;
	AddressU = BORDER;
	AddressV = BORDER;
	AddressW = BORDER;
};

texture uiMaskTex <source = "defaultMask.png"; >{ Width = BUFFER_WIDTH; Height = BUFFER_HEIGHT; Format = RGBA8; };
sampler uiMaskSampler
{
	Texture = uiMaskTex;
	AddressU = BORDER;
	AddressV = BORDER;
	AddressW = BORDER; 
};

static const float PI = 3.14159265f;

//todo would be good to have rounding param (up to circle, still bound by the rects)
float2 moveUV(float2 uv, float rounding, float4 srcRect, float4 dstRect){
	float2 result = uv;

		if (srcRect.z > 0 && dstRect.z > 0 && srcRect.w > 0 && dstRect.w > 0){
			if (uv.x >= dstRect.x && uv.x <= dstRect.x + dstRect.z){
				if (uv.y >= dstRect.y && uv.y <= dstRect.y + dstRect.w){
					//todo use rounding (range 0..1, going minimal arc to full circle or ellipse shape)

					float lx = smoothstep(dstRect.x, dstRect.x + dstRect.z, uv.x);//make uv.x on the range 0-0.25, acting as angle
					float ly = smoothstep(dstRect.y, dstRect.y + dstRect.w, uv.y);//make uv.y on the range 0-0.25, acting as angle
					float xAngle = cos(ly * 2 * PI / 4);//angle to radians (2*PI) div 4 sectors?
					float yAngle = cos(lx * 2 * PI / 4);
					float xOffset = (1 - xAngle) * dstRect.z;
					float yOffset = (1 - yAngle) * dstRect.w;


					if (true){//if (uv.x >= dstRect.x + xOffset * 2 + yOffset*2){//rounding, 1 means fully round
						float xFraction = srcRect.z / dstRect.z;
						float yFraction = srcRect.w / dstRect.w;

						result.x = srcRect.x + (uv.x - dstRect.x) * xFraction;
						result.y = srcRect.y + (uv.y - dstRect.y) * yFraction;
					}
				}
			}

		}
	return result;
}

float2 moveUVLR(float2 uv, float rounding, float4 srcRect, float4 dstRect){
	float2 movedUV = moveUV(uv, rounding, srcRect, dstRect);//move for left side, then transform rects and do on right
		movedUV = moveUV(movedUV, rounding, srcRect + float4(0.5, 0, 0, 0), dstRect + float4(0.5, 0, 0, 0));

	return movedUV;
}

float4 psRects(float4 position : SV_Position, in float2 texcoord : TEXCOORD0) : SV_Target{
	float2 movedUV = texcoord;

	if (!page0){//page0 page selection (through key toggles)
		movedUV = moveUVLR(movedUV, rounding0A, srcRect0A, dstRect0A);
		movedUV = moveUVLR(movedUV, rounding0B, srcRect0B, dstRect0B);
	}
	if (page1){//page1
		movedUV = moveUVLR(movedUV, rounding1A, srcRect1A, dstRect1A);
		movedUV = moveUVLR(movedUV, rounding1B, srcRect1B, dstRect1B);
	}
	if (page2){//page2
		movedUV = moveUVLR(movedUV, rounding2A, srcRect2A, dstRect2A);
		movedUV = moveUVLR(movedUV, rounding2B, srcRect2B, dstRect2B);
	}
	if (page3){//page3
		movedUV = moveUVLR(movedUV, rounding2A, srcRect2A, dstRect2A);
		movedUV = moveUVLR(movedUV, rounding2B, srcRect2B, dstRect2B);
	}

	//movedUV = moveUV(texcoord, srcRect, dstRect);//move for left side, then transform rects and do on right
	//movedUV = moveUV(movedUV, srcRect + float4(0.5, 0, 0, 0), dstRect + float4(0.5, 0, 0, 0));

	float4 color = tex2D(Sampler, movedUV);

		//for alpha to work we would need to mix with pixel at dest position
		float4 originalColor = tex2D(Sampler, texcoord);
		return color * alpha + originalColor * (1 - alpha);
}
float2 maskTranslation(float2 texcoord, float2 maskPosition, float2 maskScale){
	return (texcoord - maskPosition) *  (float2(2, 2) - maskScale);
}
float4 psMonoSBS(float4 position : SV_Position, in float2 texcoord : TEXCOORD0) : SV_Target{
	if (texcoord.x < 0.5)
	texcoord.x *= 2;
	else
		texcoord.x = texcoord.x * 2 - 1;
	return tex2D(Sampler, texcoord);
}
float4 psMask(float4 position : SV_Position, in float2 texcoord : TEXCOORD0) : SV_Target{
	float dynamicVignette = 0.0014; //0.0025;
	if (dynamicVignette > 0){
		if (texcoord.x < (abs(mousedelta.x) * dynamicVignette))
			discard;
		if (texcoord.x >(1 - abs(mousedelta.x) * dynamicVignette))
			discard;
		if (texcoord.y < (abs(mousedelta.y) * dynamicVignette))
			discard;
		if (texcoord.y >(1 - abs(mousedelta.y) * dynamicVignette))
			discard;
	}

	bool nonSBS = false;
	float4 originalCol = tex2D(Sampler, texcoord);

	float4 result = originalCol;
	float2 sourceCoord;
	float alphaMask = 0;

	if (nonSBS){//SBS comes from game, so ui mask needs to apply the mask on both sides (that come from the game)
		if (texcoord.x < 0.5)
			texcoord.x *= 2;
		else
			texcoord.x = texcoord.x * 2 - 1;
	}

	if (page3){//alpha channel is a bit special, doing first to avoid interference with other pages
		sourceCoord = maskTranslation(texcoord, maskPos3 + maskDstOffset, maskScale3);
		//the negated alpha means that coords outside the texture are considered mask, so clip for alpha
		//alpha is negated to make mask editing easier
		if (sourceCoord.x > 0 && sourceCoord.y > 0 &&
			sourceCoord.x < 1 && sourceCoord.y < 1)
			alphaMask = (1 - tex2D(uiMaskSampler, sourceCoord).a) * alpha;
		if (alphaMask > 0){
			float4 sourceCol = tex2D(Sampler, sourceCoord + maskSrcOffset);
				result = sourceCol * alphaMask + result * (1 - alphaMask);
		}
	}
	if (!page0){//page0 page selection (through key toggles)
		sourceCoord = maskTranslation(texcoord, maskPos0 + maskDstOffset, maskScale0);
		alphaMask = tex2D(uiMaskSampler, sourceCoord).r * alpha;//mix mask alpha with global alpha
		if (alphaMask > 0){
			float4 sourceCol = tex2D(Sampler, sourceCoord + maskSrcOffset);
				result = sourceCol * alphaMask + result * (1 - alphaMask);
		}
	}	if (page1){
		sourceCoord = maskTranslation(texcoord, maskPos1 + maskDstOffset, maskScale1);
		alphaMask = tex2D(uiMaskSampler, sourceCoord).g * alpha;
		if (alphaMask > 0){
			float4 sourceCol = tex2D(Sampler, sourceCoord + maskSrcOffset);
				result = sourceCol * alphaMask + result * (1 - alphaMask);
		}
	}
	if (page2){
		sourceCoord = maskTranslation(texcoord, maskPos2 + maskDstOffset, maskScale2);
		alphaMask = tex2D(uiMaskSampler, sourceCoord).b * alpha;
		if (alphaMask > 0){
			float4 sourceCol = tex2D(Sampler, sourceCoord + maskSrcOffset);
				result = sourceCol * alphaMask + result * (1 - alphaMask);
		}
	}

	//return tex2D(uiMaskSampler, sourceCoord);

	return result;
}

void vs(in uint id : SV_VertexID, out float4 position : SV_Position, out float2 texcoord : TEXCOORD)
{
	texcoord.x = (id == 2) ? 2.0 : 0.0;
	texcoord.y = (id == 1) ? 2.0 : 0.0;
	position = float4(texcoord * float2(2, -2.0) + float2(-1.0, 1.0), 0.0, 1.0);
}

technique moveUIRects
{
	pass p0
	{
		VertexShader = vs;
		PixelShader = psRects;
	}
}
technique moveUIMask
{
	pass p0
	{
		VertexShader = vs;
		PixelShader = psMask;
	}
}
technique monoSBS
{
	pass p0
	{
		VertexShader = vs;
		PixelShader = psMonoSBS;
	}
}